/*------------------------------------------------------------*/
/*  JAVA Class Definitions for Calendar Application           */
/*   The CalendarDay class defines an object for each day     */
/*   of the month                                             */
/*   The CalendarControl class extends the panel object for   */
/*     use suitable to maintaining/painting a calendar        */
/*   The Build4 applet displays the calendar objects          */
/*------------------------------------------------------------*/
/* Modified:                                                  */
/*   Jan 1997 - add logic to compute first, numdays values    */
/*              from month, year parameters                   */
/*            - create outer class for calendar control       */
/*              and use multiple instances of inner classes   */
/*------------------------------------------------------------*/

import java.awt.*;
import java.applet.*;
import java.util.*;

//Add a class for handling calendar daily events
//
class CalendarDay extends Object {

//    Fields
    public int    mDay;              /* day of the month       */
    public int    wDay;              /* day of week for        */
    public boolean bEvent;           /* this day has an event  */
    public String  caEvText;         /* event text             */
    public String  caEvTime;         /* event time text        */
    public String  caEvLoc;          /* event location text    */
    public String  caEvDesc;         /* event description      */
    private Rectangle caRect;        /* rectangle definition   */


//    Constructor(s)
    public CalendarDay(int x, int y, int width, int height,
                       int mday, int wday) {
        caRect = new Rectangle(x, y, width, height);
        mDay   = mday;
        wDay   = wday;
        bEvent = false;
        caEvText = null;
        caEvTime = null;
        caEvLoc  = null;
        caEvDesc = null;
    }

//    Methods

    public void  addEvent(String sText)  {
        String sToken, sKey;
        StringTokenizer sTok;

        sTok = new StringTokenizer(sText, ";");
        while (sTok.hasMoreTokens() ) {
          sToken = sTok.nextToken();
          sKey   = sToken.substring(0, 2);
          if (sKey.equals("t=") )
            addTime(sToken.substring(2));
          else if (sKey.equals("l=") )
            addLoc(sToken.substring(2));
          else if (sKey.equals("d=") )
            addDesc(sToken.substring(2));
          else
            if (!bEvent) {
              caEvText = sToken;
              bEvent = true;
            }
        }
    }

    public void addTime(String sTime) {
      if (caEvTime == null)
        caEvTime = sTime;
    }

    public void addLoc(String sLoc) {
      if (caEvLoc == null)
        caEvLoc = sLoc;
    }

    public void addDesc(String sDesc) {
      if (caEvDesc == null)
        caEvDesc = sDesc;
    }

    public void  paintDay(Graphics g, Color colBack, Color colEv, Color colDate) {

        g.setColor(Color.white);
        g.drawRect(caRect.x, caRect.y, caRect.width, caRect.height);
        if (bEvent)
          g.setColor(colBack);
        else
          g.setColor(colBack.darker());
        g.fillRect(caRect.x, caRect.y, caRect.width, caRect.height);
        g.setColor(colDate);
        g.drawString(new Integer(mDay).toString(), caRect.x+5, caRect.y+15);
        paintEvent(g, colEv);


        return;
    }

    public void  reshapeDay(int x, int y, int width, int height) {
        caRect.reshape(x, y, width, height);
    }

    public boolean inside(int x, int y) {
       return caRect.inside(x, y);
    }

    public String getText() {
        if (bEvent)
           return caEvText;
        else
           return "No event on this day";
    }

    public void  paintEvent(Graphics g, Color colEv) {

        int x1, x2, yCoord=30, yDiff=15, sLength;

        if (bEvent) {
            g.setColor(colEv);
            sLength = caEvText.length();
            x1 = 0;
            while (x1 < sLength) {
               x2 = caEvText.indexOf('.', x1);            /* next line delim */
               if (x2 > -1) {                             /* if there is one */
                   g.drawString(caEvText.substring(x1, x2),   /* draw        */
                                caRect.x+2, caRect.y+yCoord); /* substring   */
                   x1 = x2+1;                                 /* and update  */
                   yCoord += yDiff;                           /* params      */
               }
               else {                                     /* if not one      */
                   g.drawString(caEvText.substring(x1),       /* draw all of */
                                caRect.x+5, caRect.y+yCoord); /* remaining   */
                   x1 = sLength;                              /* force end   */
               }
            }
        }
        return;
    }

}

// Class to maintain panel control for calendar objects
class CalendarControl extends Panel {

// Fields
    Color     colBackground = Color.white.brighter();
    Color     colEvents = Color.black;
    Color     colDates  = Color.blue;
    Color     colSelDate = Color.yellow;
    Color     colTemp;
    TextArea  taEvents;
    String    ccMonth;

    int       i, j, k, iFirstDay=0, iNumDays=0;
    int       xFirst=10, yFirst=40, xWidth=90, yHeight=60, xSpace=05, ySpace=05;
    int       xRow, yRow, xCell, yCell;
    int       iWeekDay;
    int       iCurIx=0;
    String    sNewLine = "\r\n";
    boolean   mainMeet = false;
    boolean   boardMeet = false;
    boolean   bNoPaint  = false;
    boolean   bEvtFlag  = false;
    String    weekDays[]={" Sunday"," Monday"," Tuesday","Wednesday",
                          " Thursday","  Friday","Saturday"};

    CalendarDay  cDays[];
    TextArea  taEvent;
    String    spEvent[];


// Constructor(s)
   public CalendarControl(int cFirst, int cNum, String cMonth, TextArea ctaEvent,
                          String cspEvent[])  {
   iFirstDay = cFirst;
   iNumDays  = cNum;
   spEvent   = cspEvent;
   taEvent = ctaEvent;
   ccMonth = cMonth;

   if (spEvent[0] != null)                 /* board parameter   */
       boardMeet = true;

   taEvent.setBackground(colBackground.darker());
   taEvent.setForeground(colEvents);

   cDays = new CalendarDay[43];            /* allocate day array */

// Loop to create the initial calendar day objects
//  these are created with default rectangles which are then
//  updated by the paint routine

       iWeekDay = iFirstDay;
       i = iFirstDay + iNumDays - 1;

       for (j=iFirstDay; j<=i; j++) {
            cDays[j] = new CalendarDay(0, 0, 10, 10,
                                       j-iFirstDay+1,
                                       iWeekDay);
            if (iWeekDay >= 7)
                 iWeekDay = 1;
            else
                 iWeekDay++;
       }

       paramEvents();                 /* process parameter events */
       for (j=iFirstDay; j<i; j++)    /* process default events   */
           storeEvent(j);

   }

// Methods

// Procedure to store parameter events

   public void paramEvents() {
      int ix, ixDay;

      for (ix=0; ix<10; ix++) {
         if (spEvent[ix] != null) {
            ixDay = new Integer(spEvent[ix].substring(0, 2)).intValue() + iFirstDay - 1;
            cDays[ixDay].addEvent(spEvent[ix].substring(3));
            if (ix == 0)
               cDays[ixDay].addTime("7:15 pm");
         }
      }
   }

// Procedure to establish event text

   public void storeEvent(int ixDay) {
             int ixWeekDay = cDays[ixDay].wDay;

             if (ixWeekDay == 2) {               /* Monday Events */
                k = ixDay-iFirstDay+1-7;         /* Second Monday */
                if ((k>=1) && (k<=7)) {
                   cDays[ixDay].addEvent("Workshops");
                   cDays[ixDay].addTime("7 - 9:30 pm");
                   cDays[ixDay].addDesc("Visit the install or help desk workshops");
                }
                k = ixDay-iFirstDay+1 - 21;         /* Fourth Monday */
                if ( (k>=1) && (k<=7) ) {
                   cDays[ixDay].addEvent("SIGS");
                   cDays[ixDay].addTime("7 - 9:30 pm");
                   cDays[ixDay].addDesc("Attend Internet or Beginning User SIG");
                }
             }

             if (ixWeekDay == 7) {                 /* Saturday meeting */
                k = ixDay-iFirstDay+1 - 14;        /*  Third  */
                if ( (k>=1) && (k<=7) ) {
                   cDays[ixDay].addEvent("Main.Meeting");
                   cDays[ixDay].addTime("9am - 1pm");
                   mainMeet = true;
                }
             }

             if (ixWeekDay == 4) {             /* Wednesday board meeting */
                if ( (mainMeet) && (!boardMeet) ) {
                   cDays[ixDay].addEvent("Board.Meeting");
                   cDays[ixDay].addTime("7:15 pm");
                   cDays[ixDay].addLoc("location to be determined");
                   boardMeet = true;
                }
             }

   }

// Procedure to repaint the calendar when necessary

    public void paint(Graphics g) {
       int  minWidth;
       Dimension dimPanel = new Dimension(size());

       minWidth = (6 * xSpace) + (7 * xWidth);
       if (minWidth < dimPanel.width)
           xFirst = (dimPanel.width - minWidth) / 2;

       g.setFont(new Font("TimesRoman", Font.BOLD, 14));
       g.setColor(Color.blue);
       xCell = xFirst;
       yCell = 25;

// Draw Days of Week

       for (i=0; i<7; i++) {
         g.drawString(weekDays[i], xCell, yCell);
         xCell += xWidth + xSpace;
       }

// Draw Days Cells

       xRow  = xFirst;
       yRow  = yFirst;
       xCell = xRow;
       yCell = yRow;
       iWeekDay = 1;
       g.setColor(Color.white);

       for (j=1; j<=42; j++) {
          if ((j >= iFirstDay) && (j <= iFirstDay + iNumDays - 1)) {
             cDays[j].reshapeDay(xCell, yCell, xWidth, yHeight);
             if (j == iCurIx)
               colTemp = colSelDate;
             else
               colTemp = colBackground;
             cDays[j].paintDay(g, colTemp, colEvents, colDates);
          }

          if (iWeekDay >= 7) {
             xRow  = xFirst;
             yRow += yHeight + ySpace;
             xCell = xRow;
             yCell = yRow;
             iWeekDay = 1;
          }
          else {
             xCell += xWidth + xSpace;
             iWeekDay++;
          }
       }
    }

    public void showCalEvent(int ix) {
      iCurIx = ix;
      taEvent.setBackground(colSelDate);
      taEvent.setText(weekDays[cDays[ix].wDay-1] + " - " + ccMonth + " ");
      taEvent.appendText(new Integer(cDays[ix].mDay).toString() );
      if (cDays[ix].caEvTime != null) {
        taEvent.appendText("  (" + cDays[ix].caEvTime + ")");
      }
      if (cDays[ix].caEvLoc != null) {
        taEvent.appendText("  at  " + cDays[ix].caEvLoc);
      }
      taEvent.appendText(sNewLine + "  ");
      if (cDays[ix].bEvent) {
        taEvent.appendText(cDays[ix].caEvText.replace('.', ' '));
        if (cDays[ix].caEvDesc != null)
          taEvent.appendText(sNewLine + "    " + cDays[ix].caEvDesc);
      }
      else
        taEvent.appendText("(No event scheduled for this day)");
      taEvent.setEditable(false);
      repaint();
    }


    public boolean handleEvent(Event e) {

      int ix;

      if (e.id == Event.MOUSE_UP) {
        for (ix = iFirstDay; ix <= iFirstDay+iNumDays-1; ix++) {
           if ( cDays[ix].inside(e.x, e.y) ) {
             showCalEvent(ix);
             ix = 99;
           }
        }
      }

      return super.handleEvent(e);
    }

}

/*------------------------------------------------------------------------*/
/* This is the main applet class                                          */
/*  It is an extension of the Applet class with procedures and variables  */
/*  for the calendar processing.                                          */
/*------------------------------------------------------------------------*/

public class Build4 extends Applet {
    String pMonth = "January";
    String initMonth = null;
    String pYear  = null;
    String pBoard = null;
    String pTemp  = null;

    Panel     cardPanel = new Panel();
    Panel     bPanel    = new Panel();
    Label     lHead1    = new Label("SCOUG Events for ");
    Choice    chYear = new Choice();
    Choice    chMonth = new Choice();
    CardLayout cl = new CardLayout();
    CalendarPage    calPage[];
    String   sMonths[]={"January","February","March","April",
                        "May","June","July","August",
                        "September","October","November","December"};

    int       i, j, k;
    Applet    myApp;

    public void init() {

      calPage = new CalendarPage[12];
      myApp = this;
      pYear   = getParameter("year");

      setLayout(new BorderLayout() );

// Add control panel
      bPanel.setLayout(new FlowLayout());
      bPanel.setFont(new Font("TimesRoman", Font.BOLD, 24));

      lHead1.setForeground(Color.black);
      bPanel.add(lHead1);

      for (i=0; i<12; i++) {
        chMonth.addItem(sMonths[i]);
      }
      bPanel.add(chMonth);

      chYear.addItem(pYear);
      bPanel.add(chYear);

      add("North", bPanel);

// Add calendar page

       cardPanel.setLayout(cl);
       add("Center", cardPanel);
       addPage(0, pMonth, pYear);
       cl.show(cardPanel, pMonth);

    }

/*----------------------------------------------------------*/
/* Add Page method                                          */
/*  This method adds a new calendar page to the calendar    */
/*----------------------------------------------------------*/
    public void addPage(int ix, String sMonth, String sYear) {
      if (calPage[ix] == null) {
        calPage[ix] = new CalendarPage(myApp, sMonth, sYear, pTemp);
        cardPanel.add(sMonth, calPage[ix]);
      }
    }

/*----------------------------------------------------------*/
/* Process Choice Select event (Action override)            */
/*----------------------------------------------------------*/
    public boolean action(Event e, Object o) {

      if (e.target == chMonth) {
        if (!pMonth.equals(chMonth.getSelectedItem())) {
          pMonth = chMonth.getSelectedItem();
          addPage(chMonth.getSelectedIndex(), pMonth, pYear);
          cl.show(cardPanel, pMonth);
          lHead1.requestFocus();
        }
        return true;
      }

      return false;
    }


/*----------------------------------------------------------*/
/* System methods                                           */
/*----------------------------------------------------------*/
    public void start () {
      enable();
    }

    public void stop() {
      disable();
    }

}

/*------------------------------------------------------------------*/
/* Calendar Page class                                              */
/*  There is one instance of this class for each calendar page      */
/*  (Each page is created when first requested by user)             */
/*------------------------------------------------------------------*/

class CalendarPage extends Panel {
    String pMonth = null;
    String pYear  = null;
    String pBoard = null;
    String pEvId = null;
    String pTemp;
    Date   dFirst;

    Color     colBackground = Color.black;
    Color     colEvtxtBk = Color.blue;
    Color     colEvtxtFr = Color.white;
    Label     lHead1, lError;
    Panel     nPanel;
    Panel     bPanel;
    TextArea  taEvents;
    CalendarControl ccCal;

    int       i, j, k;
    int       iFirstDay=0, iNumDays=0;
    int       iMonth, iYear;
    int       iDayInMonth[]={31,28,31,30,31,30,31,31,30,31,30,31};
    boolean   mainMeet = false;
    boolean   boardMeet = false;
    boolean   bNoPaint  = false;
    boolean   bEvtFlag  = false;
    String    spEvent[];
    Applet    parentApp;

    public CalendarPage(Applet pApp,
                        String myMonth,
                        String myYear,
                        String myEvents) {

      parentApp = pApp;
      pMonth  = myMonth;
      pYear   = myYear;
      dFirst = new Date(pMonth + "1, " + pYear);
      pEvId  = pYear.substring(2) + pMonth.substring(0,3);

      spEvent = new String[10];
      spEvent[0] = null;
      for (i=1; i<10; i++) {
         spEvent[i] = null;
         pTemp = null;
         pTemp = parentApp.getParameter(pEvId + i);
         if (pTemp != null) {
           if (pTemp.substring(3,5).equals("r="))
             spEvent[i] = pTemp.substring(0,3) +
                          parentApp.getParameter(pTemp.substring(5));
           else
             spEvent[i] = pTemp;
         }
      }

      spEvent[0]  = parentApp.getParameter(pEvId+"b"); /* special processing for */
      if (spEvent[0] != null)                        /* board parameter        */
         boardMeet = true;

      iFirstDay = dFirst.getDay() + 1;
      iMonth    = dFirst.getMonth();
      iNumDays = iDayInMonth[iMonth];

      if (iMonth == 1) {                         /* leap year logic */
        iYear = new Integer(pYear).intValue();
        if ( (iYear % 100) == 0 ) {
          if ( (iYear % 400) == 0 ) {
            iNumDays = 29;
          }
        }
        else
          if ( (iYear % 4) == 0 ) {
            iNumDays = 29;
          }
      }

      setLayout(new BorderLayout() );

// Check for parameter errors

// Add calendar control and event text

       taEvents = new TextArea(4, 48);
       taEvents.setFont(new Font("TimesRoman", Font.BOLD, 18));


       ccCal = new CalendarControl(iFirstDay, iNumDays, pMonth, taEvents,
                                   spEvent);

       add("Center", ccCal);
       add("South", taEvents);
//     resize(700,600);

    }

}
